from dataclasses import dataclass
import os
import re
from typing import Any, Dict, List, Optional, Union

import pytest

from anyscale.job.models import JobConfig


@dataclass
class JobConfigFile:
    name: str
    expected_config: Optional[JobConfig] = None
    expected_error: Optional[str] = None

    def get_path(self) -> str:
        return os.path.join(
            os.path.dirname(__file__), "test_files/job_config_files", self.name
        )


TEST_CONFIG_FILES = [
    JobConfigFile(
        "minimal.yaml", expected_config=JobConfig(entrypoint="python test.py"),
    ),
    JobConfigFile(
        "full.yaml",
        expected_config=JobConfig(
            name="test-name-from-file",
            image_uri="docker.io/library/test:latest",
            compute_config="test-compute-config",
            working_dir="test-working-dir",
            excludes=["test"],
            requirements=["pip-install-test"],
            entrypoint="python test.py",
            max_retries=5,
            runtime_env={"env_vars": {"HELLO": "WORLD"},},
        ),
    ),
    JobConfigFile(
        "points_to_requirements_file.yaml",
        expected_config=JobConfig(
            entrypoint="python test.py", requirements="some_requirements_file.txt",
        ),
    ),
    JobConfigFile(
        "unrecognized_option.yaml",
        expected_error=re.escape(
            "__init__() got an unexpected keyword argument 'bad_option'"
        ),
    ),
]


class TestJobConfig:
    def test_invalid_entrypoint(self):
        with pytest.raises(ValueError, match="'entrypoint' cannot be empty."):
            JobConfig()

        with pytest.raises(ValueError, match="'entrypoint' cannot be empty."):
            JobConfig(entrypoint="")

        with pytest.raises(TypeError, match="'entrypoint' must be a string."):
            JobConfig(entrypoint=b"oops")

    def test_invalid_max_retries(self):
        with pytest.raises(TypeError, match="'max_retries' must be an int."):
            JobConfig(entrypoint="python test.py", max_retries="1")

        with pytest.raises(ValueError, match="'max_retries' must be >= 0."):
            JobConfig(entrypoint="python test.py", max_retries=-1)

    def test_options(self):
        config = JobConfig(entrypoint="python test.py")

        options = {
            "name": "test-name",
            "image_uri": "docker.io/libaray/test-image:latest",
            "compute_config": "test-compute-config",
            "requirements": ["pip-install-test"],
            "working_dir": ".",
            "excludes": ["some-path"],
            "max_retries": 100,
        }

        # Test setting fields one at a time.
        for option, val in options.items():
            assert config.options(**{option: val}) == JobConfig(
                entrypoint="python test.py", **{option: val}
            )

        # Test setting fields all at once.
        assert config.options(**options) == JobConfig(
            entrypoint="python test.py", **options
        )

    @pytest.mark.skip(reason="This logic will be moved to the SDK.")
    @pytest.mark.parametrize(
        ("working_dir", "excludes"),
        [
            (None, None),
            (".", ["path1/", "path2"]),
            ("s3://path.zip", ["path1/", "path2"]),
        ],
    )
    def test_override_working_dir_excludes(
        self, working_dir: Optional[str], excludes: Optional[List[str]]
    ):
        starting_runtime_envs: List[Optional[Dict[str, Any]]] = [
            None,
            {},
            {
                "excludes": ["something-else"],
                "working_dir": "s3://somewhere.zip",
                "env_vars": {"abc": "123"},
            },
        ]

        jobs = [
            JobConfig(
                entrypoint="python test.py",
                working_dir=working_dir,
                excludes=excludes,
                runtime_env=runtime_env,
            )
            for runtime_env in starting_runtime_envs
        ]

        if working_dir is None:
            assert excludes is None
            assert [job.runtime_env for job in jobs] == starting_runtime_envs
        else:
            assert isinstance(excludes, list)
            assert len(jobs) == 3
            assert jobs[0].runtime_env == {
                "working_dir": working_dir,
                "excludes": excludes,
            }
            assert jobs[1].runtime_env == {
                "working_dir": working_dir,
                "excludes": excludes,
            }
            assert jobs[2].runtime_env == {
                "working_dir": working_dir,
                # Existing `excludes` field should be extended.
                "excludes": ["something-else", *excludes],
                "env_vars": {"abc": "123"},
            }

    @pytest.mark.skip(reason="This logic will be moved to the SDK.")
    @pytest.mark.parametrize(
        ("requirements"), [(None), ("./requirements.txt"), (["req1", "req2"]),],
    )
    def test_override_requirements(
        self, requirements: Union[None, str, List[str]],
    ):
        starting_runtime_envs: List[Optional[Dict[str, Any]]] = [
            None,
            {},
            {"pip": ["something", "else"], "env_vars": {"abc": "123"},},
        ]

        jobs = [
            JobConfig(
                entrypoint="python test.py",
                requirements=requirements,
                runtime_env=runtime_env,
            )
            for runtime_env in starting_runtime_envs
        ]

        if requirements is None:
            assert [job.runtime_env for job in jobs] == starting_runtime_envs
        else:
            assert len(jobs) == 3
            assert jobs[0].runtime_env == {
                "pip": requirements,
            }
            assert jobs[0].runtime_env == {
                "pip": requirements,
            }
            assert jobs[2].runtime_env == {
                "pip": requirements,
                "env_vars": {"abc": "123"},
            }

    @pytest.mark.parametrize("config_file", TEST_CONFIG_FILES)
    def test_from_config_file(self, config_file: JobConfigFile):
        if config_file.expected_error is not None:
            with pytest.raises(Exception, match=config_file.expected_error):
                JobConfig.from_yaml(config_file.get_path())

            return

        assert config_file.expected_config == JobConfig.from_yaml(
            config_file.get_path()
        )



import re

import pytest

from anyscale._private.models import WorkloadConfig


class TestWorkloadConfig:
    def test_name(self):
        config = WorkloadConfig()
        assert config.name is None

        config = WorkloadConfig(name="my-custom-name")
        assert config.name == "my-custom-name"

        with pytest.raises(TypeError, match="'name' must be a string"):
            WorkloadConfig(name=123)

    def test_image_uri(self):
        config = WorkloadConfig()
        assert config.image_uri is None

        config = WorkloadConfig(image_uri="user/my-custom-image:1",)
        assert config.image_uri == "user/my-custom-image:1"

        with pytest.raises(TypeError, match="'image_uri' must be a string"):
            WorkloadConfig(image_uri=123)

    def test_compute_config(self):
        config = WorkloadConfig()
        assert config.compute_config is None

        config = WorkloadConfig(compute_config="my-custom-compute_config")
        assert config.compute_config == "my-custom-compute_config"

        config = WorkloadConfig(compute_config={"test": "inlined"},)
        assert config.compute_config == {"test": "inlined"}

        with pytest.raises(
            TypeError, match="'compute_config' must be a string or dictionary."
        ):
            WorkloadConfig(compute_config=123)

    def test_options(self):
        config = WorkloadConfig()

        options = {
            "name": "test-name",
            "image_uri": "docker.io/libaray/test-image:latest",
            "compute_config": "test-compute-config",
            "excludes": ["some-path"],
        }

        # Test setting fields one at a time.
        for option, val in options.items():
            assert config.options(**{option: val}) == WorkloadConfig(**{option: val})

        # Test setting fields all at once.
        assert config.options(**options) == WorkloadConfig(**options)

    def test_invalid_requirements(self):
        WorkloadConfig(requirements="test")
        WorkloadConfig(requirements=["test"])

        with pytest.raises(
            TypeError,
            match=re.escape(
                "'requirements' must be a string (file path) or list of strings."
            ),
        ):
            WorkloadConfig(requirements=1)

        with pytest.raises(
            TypeError,
            match=re.escape(
                "'requirements' must be a string (file path) or list of strings."
            ),
        ):
            WorkloadConfig(requirements=[1])

    def test_invalid_working_dir(self):
        WorkloadConfig(working_dir="test")
        with pytest.raises(TypeError, match="'working_dir' must be a string."):
            WorkloadConfig(working_dir=1)

        with pytest.raises(TypeError, match="'excludes' must be a list of strings."):
            WorkloadConfig(excludes="test")

        with pytest.raises(TypeError, match="'excludes' must be a list of strings."):
            WorkloadConfig(excludes=["test", 1])

    def test_invalid_excludes(self):
        WorkloadConfig(excludes=["test"])
        with pytest.raises(TypeError, match="'excludes' must be a list of strings."):
            WorkloadConfig(excludes="test")

        with pytest.raises(TypeError, match="'excludes' must be a list of strings."):
            WorkloadConfig(excludes=["test", 1])

    @pytest.mark.parametrize(
        ("image_uri", "valid"),
        [
            ("docker.io/libaray/ubuntu:latest", True),
            ("ubuntu:latest", False),
            ("python:3.8", False),
            ("myregistry.local:5000/testing/test-image:1.0.0", True),
            ("localhost:5000/myusername/myrepository:latest", True),
            ("localhost:5000/myusername/my/repository:latest", True),
            ("valid/withouttag", True),
            ("valid_name/withtag_and_digest:v2@sha213", True),
            ("valid_name/withtag_and_digest@sha213", True),
            ("valid_name/withtag_and_digest:@sha213", False),
            ("http://myregistry.local:5000/testing/test-image:1.0.0", False),
            (
                "us-docker.pkg.dev/anyscale-workspace-templates/workspace-templates/rag-dev-bootcamp-mar-2024:raynightly-py310",
                True,
            ),
        ],
    )
    def test_image_uri_validation(self, image_uri: str, valid: bool):
        if valid:
            assert WorkloadConfig(image_uri=image_uri).image_uri == image_uri
        else:
            with pytest.raises(
                ValueError,
                match=re.escape(
                    f"Invalid image URI: '{image_uri}'. Must be in the format: '[registry_host/]user_name/repository[:tag][@digest]'."
                ),
            ):
                WorkloadConfig(image_uri=image_uri)



from dataclasses import dataclass
import os
from typing import Any, Dict, Optional
from unittest.mock import ANY, call, Mock, patch

import pytest

from anyscale.client.openapi_client.models import WandBRunDetails
from anyscale.integrations import (
    _try_get_ray_job_id,
    get_aws_secret,
    get_endpoint,
    get_gcp_secret,
    set_wandb_project_group_env_vars,
    WANDB_API_KEY_NAME,
    wandb_get_api_key,
    WANDB_GROUP_NAME,
    WANDB_PROJECT_NAME,
    wandb_send_run_info_hook,
    wandb_setup_api_key_hook,
)
from anyscale.sdk.anyscale_client import JobsQuery
from anyscale.shared_anyscale_utils.utils.protected_string import ProtectedString


@pytest.mark.parametrize(
    "boto3_response",
    [
        {
            "ARN": "arn:aws:secretsmanager:us-west-2:188439194153:secret:my_key_name-V6JDx0",
            "Name": "my_key_name",
            "VersionId": "5a939f56-6566-42e4-9af9-673343aa61cf",
            "SecretString": "my_secret_val",
            "VersionStages": ["AWSCURRENT"],
            "ResponseMetadata": {
                "RequestId": "49887c5a-22e7-4f16-a3bb-e7344189fd1e",
                "HTTPStatusCode": 200,
                "HTTPHeaders": {
                    "x-amzn-requestid": "49887c5a-22e7-4f16-a3bb-e7344189fd1e",
                    "content-type": "application/x-amz-json-1.1",
                    "content-length": "275",
                    "date": "Tue, 08 Nov 2022 21:52:47 GMT",
                },
                "RetryAttempts": 0,
            },
        },
        {
            "ARN": "arn:aws:secretsmanager:us-west-2:188439194153:secret:my_key_name-V6JDx0",
            "Name": "my_key_name",
            "VersionId": "5a939f56-6566-42e4-9af9-673343aa61cf",
            "SecretBinary": "my_secret_val",
            "VersionStages": ["AWSCURRENT"],
            "ResponseMetadata": {
                "RequestId": "49887c5a-22e7-4f16-a3bb-e7344189fd1e",
                "HTTPStatusCode": 200,
                "HTTPHeaders": {
                    "x-amzn-requestid": "49887c5a-22e7-4f16-a3bb-e7344189fd1e",
                    "content-type": "application/x-amz-json-1.1",
                    "content-length": "275",
                    "date": "Tue, 08 Nov 2022 21:52:47 GMT",
                },
                "RetryAttempts": 0,
            },
        },
    ],
)
def test_get_aws_secret(boto3_response: Dict[str, Any]):
    mock_boto3_client = Mock(get_secret_value=Mock(return_value=boto3_response))
    mock_boto3 = Mock(client=Mock(return_value=mock_boto3_client))

    with patch.dict("sys.modules", boto3=mock_boto3):
        secret = get_aws_secret(secret_name="my_key_name", region_name="us-west-2")
        assert isinstance(secret, ProtectedString)
        assert secret._UNSAFE_DO_NOT_USE == "my_secret_val"

    mock_boto3.client.assert_called_once_with("secretsmanager", region_name="us-west-2")
    mock_boto3_client.get_secret_value.assert_called_once_with(SecretId="my_key_name")


def test_get_gcp_secret():
    mock_access_secret_version = Mock(
        return_value=Mock(payload=Mock(data=b"my_secret_val"))
    )
    mock_secret_manager_client = Mock(access_secret_version=mock_access_secret_version)
    mock_secretmanager = Mock(
        SecretManagerServiceClient=Mock(return_value=mock_secret_manager_client)
    )
    mock_google_auth = Mock(default=Mock(return_value=(ANY, "gcp_project_name")))
    mock_google = Mock(
        auth=mock_google_auth, cloud=Mock(secretmanager=mock_secretmanager)
    )

    with patch.multiple(
        "anyscale.integrations",
        try_import_gcp_secretmanager=Mock(return_value=mock_secretmanager),
    ), patch.dict("sys.modules", google=mock_google):
        assert (
            get_gcp_secret("my_key_name", key1="val1")._UNSAFE_DO_NOT_USE
            == "my_secret_val"
        )
    mock_secretmanager.SecretManagerServiceClient.assert_called_once_with(key1="val1")
    mock_google_auth.default.assert_called_once()
    mock_access_secret_version.assert_called_once_with(
        request={
            "name": "projects/gcp_project_name/secrets/my_key_name/versions/latest"
        }
    )


@pytest.mark.parametrize("cluster_id", ["mock_cluster_id", ""])
@pytest.mark.parametrize("cloud_provider", ["AWS", "GCP", "invalid_provider"])
@pytest.mark.parametrize("secret_name", ["other_api_key_name", ""])
@pytest.mark.parametrize(
    "cloud_secret_manager_key_match",
    [
        None,
        "anyscale_mock_cloud_id/mock_user_id/wandb_api_key",
        "wandb_api_key_mock_user_id",
        "other_api_key_name",
    ],
)
def test_wandb_get_api_key(
    cluster_id: Optional[str],
    cloud_provider: str,
    secret_name: Optional[str],
    cloud_secret_manager_key_match: Optional[str],
):
    if (
        cloud_secret_manager_key_match
        == "anyscale_mock_cloud_id/mock_user_id/wandb_api_key"
        and cloud_provider == "GCP"
    ):
        # Naming convention of API key for GCP has "-" instead of "/"
        cloud_secret_manager_key_match = (
            "anyscale_mock_cloud_id-mock_user_id-wandb_api_key"
        )

    mock_api_client = Mock()
    mock_cluster = Mock(cloud_id="mock_cloud_id", creator_id="mock_user_id")
    mock_api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get = Mock(
        return_value=Mock(result=mock_cluster)
    )
    mock_cloud = Mock(provider=cloud_provider, region="mock_region_name")
    mock_cloud.id = "mock_cloud_id"
    mock_api_client.get_cloud_api_v2_clouds_cloud_id_get = Mock(
        return_value=Mock(result=mock_cloud)
    )

    called_secrets = []

    def get_secret(secret_name, **kwargs):
        called_secrets.append(secret_name)
        if (
            cloud_secret_manager_key_match
            and secret_name == cloud_secret_manager_key_match
        ):
            return ProtectedString("my_secret_val")
        raise Exception(f"No key with name {secret_name} in secret manager")

    mock_get_aws_secret = Mock(side_effect=get_secret)
    mock_get_gcp_secret = Mock(side_effect=get_secret)

    with patch.dict(
        os.environ, {WANDB_API_KEY_NAME: secret_name, "ANYSCALE_SESSION_ID": cluster_id}
    ), patch.multiple(
        "anyscale.integrations",
        get_auth_api_client=Mock(return_value=Mock(api_client=mock_api_client)),
        get_aws_secret=mock_get_aws_secret,
        get_gcp_secret=mock_get_gcp_secret,
    ):
        if (
            (not cluster_id)
            or (cloud_provider not in ["AWS", "GCP"])
            or (not cloud_secret_manager_key_match)
            or (
                cloud_secret_manager_key_match == "other_api_key_name"
                and secret_name != "other_api_key_name"
            )
        ):
            with pytest.raises(Exception):  # noqa: PT011
                wandb_get_api_key()
            return

        assert wandb_get_api_key()._UNSAFE_DO_NOT_USE == "my_secret_val"
    mock_api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get.assert_called_once_with(
        cluster_id
    )
    mock_api_client.get_cloud_api_v2_clouds_cloud_id_get.assert_called_once_with(
        "mock_cloud_id"
    )

    if cloud_provider == "AWS":
        mock_get_aws_secret.assert_has_calls(
            [call(secret, region_name="mock_region_name") for secret in called_secrets]
        )
        mock_get_gcp_secret.assert_not_called()
    elif cloud_provider == "GCP":
        mock_get_gcp_secret.assert_has_calls(
            [call(secret) for secret in called_secrets]
        )
        mock_get_aws_secret.assert_not_called()


@pytest.mark.parametrize(
    "mock_ray_version", ["2.1.0", "2.2.0rc0", "2.2.0", "2.3.0rc0", "2.3.0", "3.0.0dev0"]
)
@pytest.mark.parametrize("feature_flag", [True, False])
def test_wandb_setup_api_key_hook(mock_ray_version: str, feature_flag: bool):
    mock_wandb_get_api_key = Mock(return_value=ProtectedString("my_secret_val"))
    mock_set_wandb_project_group_env_vars = Mock()
    mock_ray = Mock()
    mock_ray.__version__ = mock_ray_version
    mock_api_client = Mock()
    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        return_value=Mock(result=Mock(is_on=feature_flag))
    )

    with patch.multiple(
        "anyscale.integrations",
        wandb_get_api_key=mock_wandb_get_api_key,
        set_wandb_project_group_env_vars=mock_set_wandb_project_group_env_vars,
        get_auth_api_client=Mock(return_value=Mock(api_client=mock_api_client)),
    ), patch.dict("sys.modules", ray=mock_ray):
        if feature_flag:
            assert wandb_setup_api_key_hook() == "my_secret_val"
        else:
            assert wandb_setup_api_key_hook() is None

    if feature_flag:
        mock_wandb_get_api_key.assert_called_once_with()
        if mock_ray_version <= "2.2.0":
            mock_set_wandb_project_group_env_vars.assert_called_once_with()


@pytest.mark.parametrize("project_name_env_var", ["", "preset_project"])
@pytest.mark.parametrize("group_name_env_var", ["", "preset_project"])
@pytest.mark.parametrize("feature_flag", [True, False])
def test_set_wandb_project_group_env_vars_for_prod_job(
    project_name_env_var: str, group_name_env_var: str, feature_flag: bool
):
    os_env_var_dict = {"ANYSCALE_HA_JOB_ID": "mock_ha_job_id"}
    if project_name_env_var:
        os_env_var_dict[WANDB_PROJECT_NAME] = project_name_env_var
    if group_name_env_var:
        os_env_var_dict[WANDB_GROUP_NAME] = group_name_env_var
    mock_api_client = Mock()
    mock_production_job = Mock()
    mock_production_job.name = "mock_production_job_name"
    mock_api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get = Mock(
        return_value=Mock(result=mock_production_job)
    )
    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        return_value=Mock(result=Mock(is_on=feature_flag))
    )
    mock_slugify = Mock(side_effect=(lambda x: x))
    with patch.dict(os.environ, os_env_var_dict), patch.multiple(
        "anyscale.integrations",
        get_auth_api_client=Mock(return_value=Mock(api_client=mock_api_client)),
        slugify=mock_slugify,
    ):
        set_wandb_project_group_env_vars()

        if feature_flag:
            assert (
                project_name_env_var
                if project_name_env_var
                else os.environ[WANDB_PROJECT_NAME] == "anyscale_default_project"
            )
            assert (
                group_name_env_var
                if group_name_env_var
                else os.environ[WANDB_GROUP_NAME] == "mock_production_job_name"
            )
            mock_slugify.assert_called_once_with("mock_production_job_name")
            mock_api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_called_once_with(
                os.environ.get("ANYSCALE_HA_JOB_ID")
            )
        else:
            mock_slugify.assert_not_called()
            mock_api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_not_called()


@pytest.mark.parametrize("project_name_env_var", ["", "preset_project"])
@pytest.mark.parametrize("group_name_env_var", ["", "preset_project"])
@pytest.mark.parametrize("feature_flag", [True, False])
def test_set_wandb_project_group_env_vars_for_workspace(
    project_name_env_var: str, group_name_env_var: str, feature_flag: bool
):
    os_env_var_dict = {"ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "mock_workspace_id"}
    if project_name_env_var:
        os_env_var_dict[WANDB_PROJECT_NAME] = project_name_env_var
    if group_name_env_var:
        os_env_var_dict[WANDB_GROUP_NAME] = group_name_env_var
    mock_api_client = Mock()
    mock_workspace = Mock()
    mock_workspace.name = "mock_workspace_name"
    mock_api_client.get_workspace_api_v2_experimental_workspaces_workspace_id_get = Mock(
        return_value=Mock(result=mock_workspace)
    )
    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        return_value=Mock(result=Mock(is_on=feature_flag))
    )
    mock_slugify = Mock(side_effect=(lambda x: x))
    with patch.dict(os.environ, os_env_var_dict), patch.multiple(
        "anyscale.integrations",
        get_auth_api_client=Mock(return_value=Mock(api_client=mock_api_client)),
        slugify=mock_slugify,
    ):
        set_wandb_project_group_env_vars()
        if feature_flag:
            assert (
                project_name_env_var
                if project_name_env_var
                else os.environ[WANDB_PROJECT_NAME] == "mock_workspace_name"
            )
            assert (
                group_name_env_var
                if group_name_env_var
                else os.environ.get(WANDB_GROUP_NAME) is None
            )
            mock_slugify.assert_called_once_with("mock_workspace_name")
            mock_api_client.get_workspace_api_v2_experimental_workspaces_workspace_id_get.assert_called_once_with(
                os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID")
            )
        else:
            mock_slugify.assert_not_called()
            mock_api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_not_called()


@pytest.mark.parametrize(
    "os_env_var_dict_param", [{}, {"ANYSCALE_SESSION_ID": "mock_cluster_id"}],
)
@pytest.mark.parametrize("project_name_env_var", ["", "preset_project"])
@pytest.mark.parametrize("group_name_env_var", ["", "preset_group"])
@pytest.mark.parametrize("feature_flag", [True, False])
@pytest.mark.parametrize("try_get_ray_job_id_output", [None, "mock_ray_job_id"])
def test_set_wandb_project_group_env_vars_for_ray_job(
    os_env_var_dict_param: Dict[str, Any],
    project_name_env_var: str,
    group_name_env_var: str,
    feature_flag: bool,
    try_get_ray_job_id_output: Optional[str],
):
    os_env_var_dict = os_env_var_dict_param.copy()
    if project_name_env_var:
        os_env_var_dict[WANDB_PROJECT_NAME] = project_name_env_var
    if group_name_env_var:
        os_env_var_dict[WANDB_GROUP_NAME] = group_name_env_var

    mock_api_client = Mock()
    mock_cluster = Mock()
    mock_cluster.name = "mock_cluster_name"
    mock_api_client.get_session_api_v2_sessions_session_id_get = Mock(
        return_value=Mock(result=mock_cluster)
    )
    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        return_value=Mock(result=Mock(is_on=feature_flag))
    )
    mock_slugify = Mock(side_effect=(lambda x: x))
    mock_ray = Mock(
        get_runtime_context=Mock(
            return_value=Mock(get_job_id=Mock(return_value="mock_ray_job_id"))
        )
    )
    with patch.dict(os.environ, os_env_var_dict), patch.multiple(
        "anyscale.integrations",
        get_auth_api_client=Mock(return_value=Mock(api_client=mock_api_client)),
        slugify=mock_slugify,
        _try_get_ray_job_id=Mock(return_value=try_get_ray_job_id_output),
    ), patch.dict("sys.modules", ray=mock_ray):
        set_wandb_project_group_env_vars()
        if feature_flag:
            if try_get_ray_job_id_output:
                expected_default_project = (
                    "mock_cluster_name"
                    if os.environ.get("ANYSCALE_SESSION_ID")
                    else "anyscale_default_project"
                )
                expected_default_group = "ray_job_id_mock_ray_job_id"
                mock_slugify.assert_called_with(expected_default_group)
            else:
                expected_default_project = None
                expected_default_group = None

            assert (
                project_name_env_var
                if project_name_env_var
                else os.environ.get(WANDB_PROJECT_NAME) == expected_default_project
            )
            assert (
                group_name_env_var
                if group_name_env_var
                else os.environ.get(WANDB_GROUP_NAME) == expected_default_group
            )


@pytest.mark.parametrize("mock_wandb", [Mock(), None])
@pytest.mark.parametrize(
    "os_env_var_dict",
    [
        {
            "ANYSCALE_HA_JOB_ID": "mock_ha_job_id",
            "ANYSCALE_SESSION_ID": "mock_cluster_id",
        },
        {
            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "mock_workspace_id",
            "ANYSCALE_SESSION_ID": "mock_cluster_id",
        },
        {"ANYSCALE_SESSION_ID": "mock_cluster_id", "RAY_JOB_ID": "mock_ray_job_id"},
    ],
)
@pytest.mark.parametrize("feature_flag", [True, False])
@pytest.mark.parametrize("ray_job_exists", [True, False])
def test_wandb_send_run_info_hook(
    mock_wandb, os_env_var_dict, feature_flag, ray_job_exists
):
    @dataclass
    class MockRun:
        entity: str
        project: str
        group: Optional[str]
        config: Mock = Mock()

        get_project_url: Mock = Mock(return_value="mock_project_url")

    mock_run = MockRun(entity="mock_entity", project="mock_project", group="mock_group")
    if mock_wandb:
        mock_wandb.sdk = Mock(wandb_run=Mock(Run=MockRun))
    mock_api_client = Mock()
    mock_api_client.get_workspace_api_v2_experimental_workspaces_workspace_id_get = Mock(
        return_value=Mock(result=Mock(cluster_id="mock_cluster_id"))
    )
    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        return_value=Mock(result=Mock(is_on=feature_flag))
    )
    mock_anyscale_api_client = Mock()
    if ray_job_exists:
        mock_anyscale_api_client.search_jobs = Mock(
            return_value=Mock(results=[Mock(id="mock_ray_job_id")])
        )
    else:
        mock_anyscale_api_client.search_jobs = Mock(return_value=Mock(results=[]))
    mock_try_get_ray_job_id = Mock(return_value=os_env_var_dict.get("RAY_JOB_ID"))

    with patch.dict("sys.modules", wandb=mock_wandb), patch.dict(
        os.environ, os_env_var_dict
    ), patch.multiple(
        "anyscale.integrations",
        get_auth_api_client=Mock(
            return_value=Mock(
                api_client=mock_api_client, anyscale_api_client=mock_anyscale_api_client
            )
        ),
        _try_get_ray_job_id=mock_try_get_ray_job_id,
    ):
        if not mock_wandb and feature_flag:
            with pytest.raises(Exception):  # noqa: PT011
                wandb_send_run_info_hook(mock_run)
                return
        else:
            wandb_send_run_info_hook(mock_run)
            if os.environ.get("ANYSCALE_HA_JOB_ID") and feature_flag:
                mock_api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get.assert_called_with(
                    os.environ.get("ANYSCALE_HA_JOB_ID")
                )
                mock_api_client.put_production_job_wandb_run_details_api_v2_integrations_production_job_wandb_run_details_production_job_id_put.assert_called_once_with(
                    production_job_id=os.environ.get("ANYSCALE_HA_JOB_ID"),
                    wand_b_run_details=WandBRunDetails(
                        wandb_project_url="mock_project_url", wandb_group=mock_run.group
                    ),
                )
                assert mock_run.config.anyscale_logs == get_endpoint(
                    f"/jobs/{os.environ.get('ANYSCALE_HA_JOB_ID')}"
                )
            if os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID") and feature_flag:
                mock_api_client.put_workspace_wandb_run_details_api_v2_integrations_workspace_wandb_run_details_workspace_id_put.assert_called_once_with(
                    workspace_id=os.environ.get("ANYSCALE_EXPERIMENTAL_WORKSPACE_ID"),
                    wand_b_run_details=WandBRunDetails(
                        wandb_project_url="mock_project_url", wandb_group=mock_run.group
                    ),
                )
                assert mock_run.config.anyscale_logs == get_endpoint(
                    f"/workspaces/{os.environ.get('ANYSCALE_EXPERIMENTAL_WORKSPACE_ID')}/mock_cluster_id"
                )
            if os.environ.get("RAY_JOB_ID") and feature_flag:
                mock_anyscale_api_client.search_jobs.assert_called_with(
                    JobsQuery(
                        ray_job_id=os.environ.get("RAY_JOB_ID"),
                        cluster_id="mock_cluster_id",
                    )
                )
                if ray_job_exists:
                    mock_api_client.put_job_wandb_run_details_api_v2_integrations_job_wandb_run_details_job_id_put.assert_called_once_with(
                        job_id="mock_ray_job_id",
                        wand_b_run_details=WandBRunDetails(
                            wandb_project_url="mock_project_url",
                            wandb_group=mock_run.group,
                        ),
                    )
                    assert mock_run.config.anyscale_logs == get_endpoint(
                        "/interactive-sessions/mock_ray_job_id"
                    )
                else:
                    mock_api_client.put_job_wandb_run_details_api_v2_integrations_job_wandb_run_details_job_id_put.assert_not_called()
                    # anyscale_logs field of W&B run will not be set, but because this is a mock
                    # the field has a default mock value that should not be overridden.
                    assert mock_run.config.anyscale_logs != get_endpoint(
                        "/interactive-sessions/mock_ray_job_id"
                    )


@pytest.mark.parametrize("get_job_id_call", ["mock_ray_job_id", None])
@pytest.mark.parametrize("job_id_env_var", ["mock_ray_job_id", None])
def test_try_get_ray_job_id(
    get_job_id_call: Optional[str], job_id_env_var: Optional[str]
):
    mock_ray = Mock()
    if get_job_id_call:
        mock_ray.get_runtime_context = Mock(
            return_value=Mock(get_job_id=Mock(return_value=get_job_id_call))
        )
    else:
        mock_ray.get_runtime_context = Mock(
            return_value=Mock(get_job_id=Mock(side_effect=Exception()))
        )

    os_env_var_dict = {}
    if job_id_env_var:
        os_env_var_dict["RAY_JOB_ID"] = job_id_env_var

    with patch.dict("sys.modules", ray=mock_ray), patch.dict(
        os.environ, os_env_var_dict
    ):
        if get_job_id_call or job_id_env_var:
            assert _try_get_ray_job_id() == (get_job_id_call or job_id_env_var)
        else:
            assert _try_get_ray_job_id() is None



import copy
import pathlib
from typing import List, Optional, Tuple, Union

from common import (
    MULTI_LINE_REQUIREMENTS,
    OPENAPI_NO_VALIDATION,
    RequirementsFile,
    SINGLE_LINE_REQUIREMENTS,
    TEST_COMPUTE_CONFIG_DICT,
    TEST_CONTAINERFILE,
    TEST_REQUIREMENTS_FILES,
)
import pytest

from anyscale._private.anyscale_client import (
    FakeAnyscaleClient,
    WORKSPACE_CLUSTER_NAME_PREFIX,
)
from anyscale.client.openapi_client.models import (
    ComputeTemplate,
    CreateInternalProductionJob,
    ProductionJobConfig,
)
from anyscale.job._private.job_sdk import JobSDK
from anyscale.job.models import JobConfig


@pytest.fixture()
def sdk_with_fake_client() -> Tuple[JobSDK, FakeAnyscaleClient]:
    fake_client = FakeAnyscaleClient()
    return JobSDK(client=fake_client), fake_client


class TestSubmit:
    def test_basic(self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]):
        sdk, fake_client = sdk_with_fake_client

        config = JobConfig(entrypoint="python hello.py", name="test-job-name",)
        sdk.submit(config)
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
                compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
            ),
        )

    def test_default_name(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client

        config = JobConfig(entrypoint="python hello.py",)
        sdk.submit(config)
        assert fake_client.submitted_job.name
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name=fake_client.submitted_job.name,
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
                compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
            ),
        )

    def test_custom_image_from_containerfile(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_containerfile = pathlib.Path(TEST_CONTAINERFILE).read_text()
        fake_client.set_containerfile_mapping(fake_containerfile, "bld_1234")

        config = JobConfig(
            entrypoint="python hello.py",
            name="test-job-name",
            containerfile=TEST_CONTAINERFILE,
        )
        sdk.submit(config)
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id="bld_1234",
                compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
            ),
        )

    def test_custom_image_from_image_uri(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.set_image_uri_mapping(
            "docker.io/user/my-custom-image:latest", "bld_123"
        )

        config = JobConfig(
            entrypoint="python hello.py",
            name="test-job-name",
            image_uri="docker.io/user/my-custom-image:latest",
        )
        sdk.submit(config)
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id="bld_123",
                compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
            ),
        )

        with pytest.raises(
            ValueError,
            match="The image_uri 'docker.io/user/does-not-exist:latest' does not exist.",
        ):
            sdk.submit(config.options(image_uri="docker.io/user/does-not-exist:latest"))

    def test_custom_compute_config_name(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client

        fake_client.add_compute_config(
            ComputeTemplate(
                id="compute_id123",
                name="my-custom-compute-config",
                local_vars_configuration=OPENAPI_NO_VALIDATION,
            )
        )

        config = JobConfig(
            entrypoint="python hello.py",
            name="test-job-name",
            compute_config="my-custom-compute-config",
        )
        sdk.submit(config)
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
                compute_config_id="compute_id123",
            ),
        )

        with pytest.raises(
            ValueError, match="The compute config 'does-not-exist' does not exist."
        ):
            sdk.submit(config.options(compute_config="does-not-exist"))

    def test_custom_compute_config_dict(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client

        # Specify a compute config as a dictionary (anonymous).
        config = JobConfig(
            entrypoint="python hello.py",
            name="test-job-name",
            compute_config=TEST_COMPUTE_CONFIG_DICT,
        )
        sdk.submit(config)
        anonymous_compute_config_id = fake_client.submitted_job.config.compute_config_id
        anonymous_compute_config = fake_client.get_compute_config(
            anonymous_compute_config_id
        )
        assert anonymous_compute_config.anonymous
        assert anonymous_compute_config.id == anonymous_compute_config_id
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=1,
                build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
                compute_config_id=anonymous_compute_config_id,
            ),
        )

        # Test an invalid dict.
        with pytest.raises(RuntimeError, match="Invalid compute config"):
            sdk.submit(
                JobConfig(
                    entrypoint="python hello.py",
                    name="test-job-name",
                    compute_config={"bad": "config"},
                )
            )

    def test_max_retries(self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]):
        sdk, fake_client = sdk_with_fake_client

        config = JobConfig(
            entrypoint="python hello.py", name="test-job-name", max_retries=10,
        )
        sdk.submit(config)
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.DEFAULT_PROJECT_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env=None,
                max_retries=10,
                build_id=fake_client.DEFAULT_CLUSTER_ENV_BUILD_ID,
                compute_config_id=fake_client.DEFAULT_CLUSTER_COMPUTE_ID,
            ),
        )


class TestDeployWorkspaceDefaults:
    def test_name_defaults_to_workspace_name(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client

        # Happy path: workspace cluster name has the expected prefix.
        fake_client.set_inside_workspace(
            True,
            cluster_name=WORKSPACE_CLUSTER_NAME_PREFIX + "super-special-workspace-name",
        )
        sdk.submit(JobConfig(entrypoint="python hello.py"))
        assert fake_client.submitted_job.name == "job-super-special-workspace-name"

        # Defensive path: workspace cluster name doesn't have the expected prefix.
        fake_client.set_inside_workspace(
            True, cluster_name="not-sure-how-this-happened"
        )
        sdk.submit(JobConfig(entrypoint="python hello.py"))
        assert fake_client.submitted_job.name == "job-not-sure-how-this-happened"

    def test_pick_up_cluster_configs(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.set_inside_workspace(True)

        sdk.submit(
            JobConfig(
                entrypoint="python hello.py",
                name="test-job-name",
                working_dir="s3://remote.zip",
            )
        )
        assert fake_client.submitted_job == CreateInternalProductionJob(
            name="test-job-name",
            project_id=fake_client.WORKSPACE_PROJECT_ID,
            workspace_id=fake_client.WORKSPACE_ID,
            config=ProductionJobConfig(
                entrypoint="python hello.py",
                runtime_env={"working_dir": "s3://remote.zip"},
                max_retries=1,
                build_id=fake_client.WORKSPACE_CLUSTER_ENV_BUILD_ID,
                compute_config_id=fake_client.WORKSPACE_CLUSTER_COMPUTE_ID,
            ),
        )


class TestOverrideApplicationRuntimeEnvs:
    @pytest.mark.parametrize("excludes_override", [None, ["override"]])
    @pytest.mark.parametrize(
        "working_dir_override", [None, "./some-local-path", "s3://some-remote-path.zip"]
    )
    @pytest.mark.parametrize("inside_workspace", [False, True])
    def test_override_working_dir_excludes(
        self,
        sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
        inside_workspace: bool,
        working_dir_override: Optional[str],
        excludes_override: Optional[List[str]],
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.set_inside_workspace(inside_workspace)

        base_job = JobConfig(
            name="test-job-name",
            entrypoint="python hello.py",
            working_dir=working_dir_override,
            excludes=excludes_override,
        )

        jobs = [
            base_job.options(runtime_env=None),
            base_job.options(runtime_env={}),
            base_job.options(
                runtime_env={"working_dir": "other-path/", "excludes": ["existing"],}
            ),
            # `None` should avoid workspace overriding.
            base_job.options(runtime_env={"working_dir": None, "excludes": None,}),
        ]

        submitted_runtime_envs = []
        for job in jobs:
            sdk.submit(job)
            submitted_runtime_envs.append(fake_client.submitted_job.config.runtime_env)

        cloud_id = (
            FakeAnyscaleClient.WORKSPACE_CLOUD_ID
            if inside_workspace
            else FakeAnyscaleClient.DEFAULT_CLOUD_ID
        )
        cwd_uri = fake_client.upload_local_dir_to_cloud_storage(".", cloud_id=cloud_id)
        if working_dir_override is None and not inside_workspace:
            assert (
                submitted_runtime_envs[0] is None
                or "working_dir" not in submitted_runtime_envs[0]
            )
            assert "working_dir" not in submitted_runtime_envs[1]
            assert submitted_runtime_envs[2][
                "working_dir"
            ] == fake_client.upload_local_dir_to_cloud_storage(
                "other-path/", cloud_id=cloud_id,
            )
            assert submitted_runtime_envs[3]["working_dir"] is None
        elif working_dir_override is None and inside_workspace:
            assert submitted_runtime_envs[0]["working_dir"] == cwd_uri
            assert submitted_runtime_envs[1]["working_dir"] == cwd_uri
            assert submitted_runtime_envs[2][
                "working_dir"
            ] == fake_client.upload_local_dir_to_cloud_storage(
                "other-path/", cloud_id=cloud_id,
            )
            assert submitted_runtime_envs[3]["working_dir"] is None
        elif working_dir_override is not None and working_dir_override.startswith("s3"):
            assert submitted_runtime_envs[0]["working_dir"] == working_dir_override
            assert submitted_runtime_envs[1]["working_dir"] == working_dir_override
            assert submitted_runtime_envs[2]["working_dir"] == working_dir_override
            assert submitted_runtime_envs[3]["working_dir"] == working_dir_override
        else:
            override_uri = fake_client.upload_local_dir_to_cloud_storage(
                "./some-local-path", cloud_id=cloud_id
            )
            assert submitted_runtime_envs[0]["working_dir"] == override_uri
            assert submitted_runtime_envs[1]["working_dir"] == override_uri
            assert submitted_runtime_envs[2]["working_dir"] == override_uri
            assert submitted_runtime_envs[3]["working_dir"] == override_uri

        if excludes_override is None:
            assert (
                submitted_runtime_envs[0] is None
                or "excludes" not in submitted_runtime_envs[0]
            )
            assert "excludes" not in submitted_runtime_envs[1]
            assert submitted_runtime_envs[2]["excludes"] == ["existing"]
            assert submitted_runtime_envs[3]["excludes"] is None
        else:
            assert submitted_runtime_envs[0]["excludes"] == ["override"]
            assert submitted_runtime_envs[1]["excludes"] == ["override"]
            assert submitted_runtime_envs[2]["excludes"] == [
                "existing",
                "override",
            ]
            assert submitted_runtime_envs[3]["excludes"] == ["override"]

    @pytest.mark.parametrize(
        "requirements_override",
        [None, MULTI_LINE_REQUIREMENTS.get_path(), ["override"]],
    )
    @pytest.mark.parametrize("workspace_tracking_enabled", [False, True])
    @pytest.mark.parametrize("inside_workspace", [False, True])
    def test_override_requirements(
        self,
        sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
        inside_workspace: bool,
        workspace_tracking_enabled: bool,
        requirements_override: Union[None, str, List[str]],
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.set_inside_workspace(
            inside_workspace,
            requirements_path=SINGLE_LINE_REQUIREMENTS.get_path()
            if workspace_tracking_enabled
            else None,
        )

        base_job = JobConfig(
            name="test-job-name",
            entrypoint="python hello.py",
            requirements=requirements_override,
        )

        jobs = [
            base_job.options(runtime_env=None),
            base_job.options(runtime_env={}),
            base_job.options(runtime_env={"pip": MULTI_LINE_REQUIREMENTS.get_path(),}),
            base_job.options(runtime_env={"pip": ["testabc", "test123"],}),
            # `None` should avoid workspace overriding.
            base_job.options(runtime_env={"pip": None,}),
        ]

        submitted_runtime_envs = []
        for job in jobs:
            sdk.submit(job)
            submitted_runtime_envs.append(fake_client.submitted_job.config.runtime_env)

        if isinstance(requirements_override, str):
            # Override with a file.
            pass
        elif isinstance(requirements_override, list):
            # Override with a list.
            pass
        elif inside_workspace and workspace_tracking_enabled:
            # Workspace default.
            expected_workspace_pip = SINGLE_LINE_REQUIREMENTS.expected_pip_list
            assert submitted_runtime_envs[0]["pip"] == expected_workspace_pip
            assert submitted_runtime_envs[1]["pip"] == expected_workspace_pip
            assert (
                submitted_runtime_envs[2]["pip"]
                == MULTI_LINE_REQUIREMENTS.expected_pip_list
            )
            assert submitted_runtime_envs[3]["pip"] == ["testabc", "test123"]
            assert submitted_runtime_envs[4]["pip"] is None
        else:
            # No overrides.
            assert (
                submitted_runtime_envs[0] is None
                or "pip" not in submitted_runtime_envs[0]
            )
            assert "pip" not in submitted_runtime_envs[1]
            assert (
                submitted_runtime_envs[2]["pip"]
                == MULTI_LINE_REQUIREMENTS.expected_pip_list
            )
            assert submitted_runtime_envs[3]["pip"] == ["testabc", "test123"]
            assert submitted_runtime_envs[4]["pip"] is None


class TestDeployUploadDirs:
    @pytest.mark.parametrize("inside_workspace", [False, True])
    def test_upload_basic_working_dir(
        self,
        sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
        inside_workspace: bool,
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.set_inside_workspace(inside_workspace)

        config = JobConfig(entrypoint="python test.py", working_dir=".",)
        original_config = copy.deepcopy(config)

        sdk.submit(config)
        # The original config should not be modified.
        assert config == original_config

        # Check that the correct cloud_id was used for the upload.
        expected_cloud_id = (
            FakeAnyscaleClient.WORKSPACE_CLOUD_ID
            if inside_workspace
            else FakeAnyscaleClient.DEFAULT_CLOUD_ID
        )
        assert fake_client.submitted_job.config.runtime_env["working_dir"].startswith(
            FakeAnyscaleClient.CLOUD_BUCKET.format(cloud_id=expected_cloud_id)
        )

    def test_upload_uses_cloud_from_compute_config(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
    ):
        sdk, fake_client = sdk_with_fake_client
        fake_client.add_compute_config(
            ComputeTemplate(
                id="fake-compute-config-id",
                name="fake-compute-config",
                local_vars_configuration=OPENAPI_NO_VALIDATION,
            )
        )
        fake_client.set_compute_config_id_to_cloud_id_mapping(
            "fake-compute-config-id", "compute-config-cloud-id"
        )

        config = JobConfig(
            entrypoint="python test.py",
            working_dir=".",
            compute_config="fake-compute-config",
        )
        sdk.submit(config)

        # Check that the correct cloud_id was used for the upload.
        expected_cloud_id = "compute-config-cloud-id"
        assert fake_client.submitted_job.config.runtime_env["working_dir"].startswith(
            FakeAnyscaleClient.CLOUD_BUCKET.format(cloud_id=expected_cloud_id)
        )

    def test_upload_with_no_local_dirs(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        """Configs should be left unchanged if there are no local dirs."""
        sdk, fake_client = sdk_with_fake_client

        basic_config = JobConfig(entrypoint="python test.py",)
        sdk.submit(basic_config)
        assert fake_client.submitted_job.config.runtime_env is None

        config_with_requirements = JobConfig(
            entrypoint="python test.py", requirements=["pip-install-test"],
        )
        sdk.submit(config_with_requirements)
        assert fake_client.submitted_job.config.runtime_env == {
            "pip": ["pip-install-test"]
        }

        config_with_runtime_env = JobConfig(
            entrypoint="python test.py", runtime_env={"env_vars": {"foo": "bar"}},
        )
        sdk.submit(config_with_runtime_env)
        assert fake_client.submitted_job.config.runtime_env == {
            "env_vars": {"foo": "bar"},
        }

    def test_no_upload_remote_working_dir(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client
        config = JobConfig(
            entrypoint="python test.py", working_dir="s3://some-remote-uri.zip",
        )

        sdk.submit(config)
        assert (
            fake_client.submitted_job.config.runtime_env["working_dir"]
            == "s3://some-remote-uri.zip"
        )

    def test_upload_local_py_modules(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        sdk, fake_client = sdk_with_fake_client
        config = JobConfig(
            entrypoint="python test.py",
            runtime_env={
                "py_modules": [
                    # Should be left alone.
                    "s3://some-remote-uri.zip",
                    # Should be uploaded.
                    "local-path",
                ],
            },
        )
        sdk.submit(config)
        assert (
            fake_client.submitted_job.config.runtime_env["py_modules"][0]
            == "s3://some-remote-uri.zip"
        )
        assert fake_client.submitted_job.config.runtime_env["py_modules"][1].startswith(
            FakeAnyscaleClient.CLOUD_BUCKET.format(
                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
            )
        )

    def test_upload_caching(
        self, sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient]
    ):
        """The same directory should only by uploaded once."""
        sdk, fake_client = sdk_with_fake_client
        config = JobConfig(
            entrypoint="python test.py",
            runtime_env={"working_dir": ".", "py_modules": [".", "other-dir"],},
        )
        sdk.submit(config)
        assert fake_client.submitted_job.config.runtime_env["working_dir"].startswith(
            FakeAnyscaleClient.CLOUD_BUCKET.format(
                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
            )
        )
        assert (
            fake_client.submitted_job.config.runtime_env["working_dir"]
            == fake_client.submitted_job.config.runtime_env["py_modules"][0]
        )

        assert fake_client.submitted_job.config.runtime_env["py_modules"][1].startswith(
            FakeAnyscaleClient.CLOUD_BUCKET.format(
                cloud_id=FakeAnyscaleClient.DEFAULT_CLOUD_ID
            )
        )
        assert (
            fake_client.submitted_job.config.runtime_env["py_modules"][0]
            != fake_client.submitted_job.config.runtime_env["py_modules"][1]
        )


class TestLoadRequirementsFiles:
    @pytest.mark.parametrize("requirements", [None, *TEST_REQUIREMENTS_FILES])
    @pytest.mark.parametrize("in_runtime_env", [False, True])
    def test_override_requirements_file(
        self,
        sdk_with_fake_client: Tuple[JobSDK, FakeAnyscaleClient],
        requirements: Optional[RequirementsFile],
        in_runtime_env: bool,
    ):
        sdk, fake_client = sdk_with_fake_client

        base_job = JobConfig(entrypoint="python hello.py",)

        jobs = [
            base_job.options(runtime_env=None),
            base_job.options(runtime_env={}),
            base_job.options(
                runtime_env={
                    "env_vars": {"abc": "123"},
                    "working_dir": "s3://somewhere.zip",
                }
            ),
        ]

        submitted_runtime_envs = []
        for i, job in list(enumerate(jobs)):
            if requirements is not None:
                if in_runtime_env:
                    jobs[i] = job.options(
                        runtime_env={
                            "pip": requirements.get_path(),
                            **(job.runtime_env or {}),
                        },
                    )
                else:
                    jobs[i] = job.options(requirements=requirements.get_path(),)

        if requirements is not None and requirements.expected_pip_list is None:
            for job in jobs:
                with pytest.raises(FileNotFoundError):
                    sdk.submit(job)

            return
        else:
            for job in jobs:
                sdk.submit(job)
                submitted_runtime_envs.append(
                    fake_client.submitted_job.config.runtime_env
                )

        if requirements is None:
            assert submitted_runtime_envs == [job.runtime_env for job in jobs]
        else:
            assert submitted_runtime_envs[0] == {
                "pip": requirements.expected_pip_list,
            }

            assert submitted_runtime_envs[1] == {
                "pip": requirements.expected_pip_list,
            }
            assert submitted_runtime_envs[2] == {
                "env_vars": {"abc": "123"},
                "working_dir": "s3://somewhere.zip",
                "pip": requirements.expected_pip_list,
            }



# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class StaticSSOConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'idp_entity_id': 'str',
        'idp_sso_url': 'str',
        'idp_sso_binding': 'str',
        'idp_x509cert': 'str'
    }

    attribute_map = {
        'idp_entity_id': 'idp_entity_id',
        'idp_sso_url': 'idp_sso_url',
        'idp_sso_binding': 'idp_sso_binding',
        'idp_x509cert': 'idp_x509cert'
    }

    def __init__(self, idp_entity_id=None, idp_sso_url=None, idp_sso_binding=None, idp_x509cert=None, local_vars_configuration=None):  # noqa: E501
        """StaticSSOConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._idp_entity_id = None
        self._idp_sso_url = None
        self._idp_sso_binding = None
        self._idp_x509cert = None
        self.discriminator = None

        self.idp_entity_id = idp_entity_id
        self.idp_sso_url = idp_sso_url
        self.idp_sso_binding = idp_sso_binding
        self.idp_x509cert = idp_x509cert

    @property
    def idp_entity_id(self):
        """Gets the idp_entity_id of this StaticSSOConfig.  # noqa: E501

        Static configuration for the SAML 2.0 identity provider's entity id.  # noqa: E501

        :return: The idp_entity_id of this StaticSSOConfig.  # noqa: E501
        :rtype: str
        """
        return self._idp_entity_id

    @idp_entity_id.setter
    def idp_entity_id(self, idp_entity_id):
        """Sets the idp_entity_id of this StaticSSOConfig.

        Static configuration for the SAML 2.0 identity provider's entity id.  # noqa: E501

        :param idp_entity_id: The idp_entity_id of this StaticSSOConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and idp_entity_id is None:  # noqa: E501
            raise ValueError("Invalid value for `idp_entity_id`, must not be `None`")  # noqa: E501

        self._idp_entity_id = idp_entity_id

    @property
    def idp_sso_url(self):
        """Gets the idp_sso_url of this StaticSSOConfig.  # noqa: E501

        Static configuration for the SAML 2.0 identity provider's url.  # noqa: E501

        :return: The idp_sso_url of this StaticSSOConfig.  # noqa: E501
        :rtype: str
        """
        return self._idp_sso_url

    @idp_sso_url.setter
    def idp_sso_url(self, idp_sso_url):
        """Sets the idp_sso_url of this StaticSSOConfig.

        Static configuration for the SAML 2.0 identity provider's url.  # noqa: E501

        :param idp_sso_url: The idp_sso_url of this StaticSSOConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and idp_sso_url is None:  # noqa: E501
            raise ValueError("Invalid value for `idp_sso_url`, must not be `None`")  # noqa: E501

        self._idp_sso_url = idp_sso_url

    @property
    def idp_sso_binding(self):
        """Gets the idp_sso_binding of this StaticSSOConfig.  # noqa: E501

        Static configuration for the SAML 2.0 identity provider's sso binding.  # noqa: E501

        :return: The idp_sso_binding of this StaticSSOConfig.  # noqa: E501
        :rtype: str
        """
        return self._idp_sso_binding

    @idp_sso_binding.setter
    def idp_sso_binding(self, idp_sso_binding):
        """Sets the idp_sso_binding of this StaticSSOConfig.

        Static configuration for the SAML 2.0 identity provider's sso binding.  # noqa: E501

        :param idp_sso_binding: The idp_sso_binding of this StaticSSOConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and idp_sso_binding is None:  # noqa: E501
            raise ValueError("Invalid value for `idp_sso_binding`, must not be `None`")  # noqa: E501

        self._idp_sso_binding = idp_sso_binding

    @property
    def idp_x509cert(self):
        """Gets the idp_x509cert of this StaticSSOConfig.  # noqa: E501

        Static configuration for the SAML 2.0 identity provider's x509cert  # noqa: E501

        :return: The idp_x509cert of this StaticSSOConfig.  # noqa: E501
        :rtype: str
        """
        return self._idp_x509cert

    @idp_x509cert.setter
    def idp_x509cert(self, idp_x509cert):
        """Sets the idp_x509cert of this StaticSSOConfig.

        Static configuration for the SAML 2.0 identity provider's x509cert  # noqa: E501

        :param idp_x509cert: The idp_x509cert of this StaticSSOConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and idp_x509cert is None:  # noqa: E501
            raise ValueError("Invalid value for `idp_x509cert`, must not be `None`")  # noqa: E501

        self._idp_x509cert = idp_x509cert

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StaticSSOConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StaticSSOConfig):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional
from unittest.mock import ANY, Mock

import pytest

from anyscale.cluster_compute import (
    get_cluster_compute_from_name,
    get_default_cluster_compute,
    get_selected_cloud_id_or_default,
    parse_cluster_compute_name_version,
    register_compute_template,
)
from anyscale.sdk.anyscale_client import ArchiveStatus, CreateComputeTemplate
from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
    ClusterComputeConfig,
)
from anyscale.sdk.anyscale_client.models.compute_template_query import (
    ComputeTemplateQuery,
)


def test_get_default_cluster_compute():
    mock_api_client = Mock()
    mock_anyscale_api_client = Mock()
    mock_anyscale_api_client.get_default_compute_config = Mock(
        return_value=Mock(result="mock_config_obj")
    )
    mock_default_config_obj = {
        "cloud_id": "cld_1",
        "max_workers": 10,
        "region": "us-west-2",
        "allowed_azs": ["us-west-2a", "us-west-2b"],
        "head_node_type": {
            "name": "head-1",
            "instance_type": "m5.2xlarge",
            "aws_advanced_configurations": None,
            "gcp_advanced_configurations": None,
            "resources": None,
        },
        "worker_node_types": [],
    }
    mock_anyscale_api_client.get_default_compute_config = Mock(
        return_value=Mock(result=mock_default_config_obj)
    )
    mock_api_client.create_compute_template_api_v2_compute_templates_post = Mock(
        return_value=Mock(result="mock_compute_template")
    )
    mock_api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post = Mock(
        return_value=Mock(result=Mock(id="test_cloud_id"))
    )

    assert (
        get_default_cluster_compute(
            "mock_cloud_name", None, mock_api_client, mock_anyscale_api_client
        )
        == "mock_compute_template"
    )
    mock_anyscale_api_client.get_default_compute_config.assert_called_once_with(
        "test_cloud_id"
    )


def test_register_compute_template():
    mock_api_client = Mock()
    mock_api_client.create_compute_template_api_v2_compute_templates_post = Mock(
        return_value=Mock(result=Mock(id="mock_template_id"))
    )

    assert (
        register_compute_template("mock_template_obj", mock_api_client).id
        == "mock_template_id"
    )
    mock_api_client.create_compute_template_api_v2_compute_templates_post.assert_called_once_with(
        create_compute_template=CreateComputeTemplate(
            name=ANY, config="mock_template_obj", anonymous=True,
        )
    )


@pytest.mark.parametrize("version", [None, 2])
def test_get_cluster_compute_from_name(version: Optional[int]):
    mock_api_client = Mock()
    mock_api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
        return_value=Mock(results=[Mock(id="mock_id")])
    )
    assert (
        get_cluster_compute_from_name("mock_name", mock_api_client, version).id
        == "mock_id"
    )

    mock_api_client.search_compute_templates_api_v2_compute_templates_search_post.assert_called_once_with(
        ComputeTemplateQuery(
            orgwide=True,
            name={"equals": "mock_name"},
            include_anonymous=True,
            archive_status=ArchiveStatus.ALL,
            version=version,
        )
    )


@pytest.mark.parametrize("enable_compute_config_versioning", [True, False])
def test_get_cluster_compute_from_name_version_conflict_detection(
    enable_compute_config_versioning: bool,
):
    mock_api_client = Mock()
    mock_api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
        return_value=Mock(results=[Mock(id="mock_id")])
    )

    def mock_feature_flag_api(feature_flag: str):
        if feature_flag == "compute-config-versioning":
            return Mock(result=Mock(is_on=enable_compute_config_versioning))

    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        side_effect=mock_feature_flag_api
    )

    if enable_compute_config_versioning:
        # test provide version in both name and version field
        with pytest.raises(ValueError) as e:
            get_cluster_compute_from_name("mock_name:2", mock_api_client, version=3)
            e.match(
                "Please provide either version in the name or as a separate argument, not both."
            )
    else:
        assert (
            get_cluster_compute_from_name("mock_name:2", mock_api_client, version=3).id
            == "mock_id"
        )
        mock_api_client.search_compute_templates_api_v2_compute_templates_search_post.assert_called_once_with(
            ComputeTemplateQuery(
                orgwide=True,
                name={"equals": "mock_name:2"},
                include_anonymous=True,
                archive_status=ArchiveStatus.ALL,
                version=3,
            )
        )


@pytest.mark.parametrize(
    (
        "enable_compute_config_versioning",
        "compute_config_name_version",
        "deprecated_version_field",
        "expected_name",
        "expected_version",
    ),
    [
        (True, "mock_name:2", None, "mock_name", 2),
        (False, "mock_name:2", None, "mock_name:2", None),
        (True, "mock_name", 2, "mock_name", 2),
        (False, "mock_name", 2, "mock_name", 2),
        (True, "mock_name", None, "mock_name", None),
        (False, "mock_name", None, "mock_name", None),
    ],
)
def test_get_cluster_compute_from_name_version_with_version_provided_in_name(
    enable_compute_config_versioning: bool,
    compute_config_name_version: str,
    deprecated_version_field: Optional[int],
    expected_name: str,
    expected_version: Optional[int],
):
    mock_api_client = Mock()
    mock_api_client.search_compute_templates_api_v2_compute_templates_search_post = Mock(
        return_value=Mock(results=[Mock(id="mock_id")])
    )

    def mock_feature_flag_api(feature_flag: str):
        if feature_flag == "compute-config-versioning":
            return Mock(result=Mock(is_on=enable_compute_config_versioning))

    mock_api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get = Mock(
        side_effect=mock_feature_flag_api
    )
    assert (
        get_cluster_compute_from_name(
            compute_config_name_version,
            mock_api_client,
            version=deprecated_version_field,
        ).id
        == "mock_id"
    )

    mock_api_client.search_compute_templates_api_v2_compute_templates_search_post.assert_called_once_with(
        ComputeTemplateQuery(
            orgwide=True,
            name={"equals": expected_name},
            include_anonymous=True,
            archive_status=ArchiveStatus.ALL,
            version=expected_version,
        )
    )


def test_get_selected_cloud_id_or_default():
    mock_api_client = Mock()
    mock_anyscale_api_client = Mock()

    mock_cloud = Mock(id="mock_cloud_id")
    mock_cloud.name = "mock_cloud_name"

    # Test getting cloud from cluster compute id
    assert (
        get_selected_cloud_id_or_default(
            api_client=mock_api_client,
            anyscale_api_client=mock_anyscale_api_client,
            cluster_compute_id="mock_cluster_compute_id",
        )
        == mock_anyscale_api_client.get_cluster_compute(
            "mock_cluster_compute_id"
        ).result.config.cloud_id
    )

    # Test getting cloud from cluster compute config
    cluster_compute_dict = {
        "cloud_id": "mock_cloud_id",
        "max_workers": 10,
        "region": "us-west-2",
        "allowed_azs": ["us-west-2a", "us-west-2b"],
        "head_node_type": {
            "name": "head-1",
            "instance_type": "m5.2xlarge",
            "aws_advanced_configurations": None,
            "gcp_advanced_configurations": None,
            "resources": None,
        },
        "worker_node_types": [],
    }
    cluster_compute_config = ClusterComputeConfig(**cluster_compute_dict)
    assert (
        get_selected_cloud_id_or_default(
            api_client=mock_api_client,
            anyscale_api_client=mock_anyscale_api_client,
            cluster_compute_config=cluster_compute_config,
        )
        == "mock_cloud_id"
    )

    # Test getting cloud from cloud_id
    mock_api_client.get_cloud_api_v2_clouds_cloud_id_get = Mock(
        return_value=Mock(result=mock_cloud)
    )
    assert (
        get_selected_cloud_id_or_default(
            api_client=mock_api_client,
            anyscale_api_client=mock_anyscale_api_client,
            cloud_id="mock_cloud_id",
        )
        == "mock_cloud_id"
    )
    mock_api_client.get_cloud_api_v2_clouds_cloud_id_get.assert_called_once_with(
        cloud_id="mock_cloud_id"
    )

    # Test getting cloud from cloud_name
    mock_api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post = Mock(
        return_value=Mock(result=mock_cloud)
    )
    assert (
        get_selected_cloud_id_or_default(
            api_client=mock_api_client,
            anyscale_api_client=mock_anyscale_api_client,
            cloud_name="mock_cloud_name",
        )
        == "mock_cloud_id"
    )
    mock_api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post.assert_called_once_with(
        cloud_name_options={"name": "mock_cloud_name"}
    )

    # Test getting cloud from default cluster compute
    mock_anyscale_api_client.get_default_cluster_compute = Mock(
        return_value=Mock(result=Mock(config=Mock(cloud_id="mock_default_cloud_id")))
    )
    assert (
        get_selected_cloud_id_or_default(
            api_client=mock_api_client, anyscale_api_client=mock_anyscale_api_client
        )
        == "mock_default_cloud_id"
    )
    mock_anyscale_api_client.get_default_cluster_compute.assert_called_once_with()


@pytest.mark.parametrize(
    ("cluster_compute_name_version", "expected_name", "expected_version", "error"),
    [
        ("name", "name", None, False),
        ("1", "1", None, False),
        ("name:0", None, None, True),
        ("name:1", "name", 1, False),
        ("name:2", "name", 2, False),
        ("name:3", "name", 3, False),
        ("name:4", "name", 4, False),
        ("name:5", "name", 5, False),
        ("name:6", "name", 6, False),
        ("name:7", "name", 7, False),
        ("name:8", "name", 8, False),
        ("name:9", "name", 9, False),
        ("name:10", "name", 10, False),
        ("10:10", "10", 10, False),
        ("name:0010", None, None, True),
        ("name with spaces:11", "name with spaces", 11, False),
        ("name with    more   spaces:11", "name with    more   spaces", 11, False),
        ("name-with-tailing-spaces :11", None, None, True),
        ("name-with-tailing-spaces ", None, None, True),
        (" name-with-leading-spaces:11", None, None, True),
        (" name-with-leading-spaces", None, None, True),
        (" name-with-tailing-and-leading-spaces :11", None, None, True),
        (" name-with-tailing-and-leading-spaces ", None, None, True),
        (
            "name-with-special-chars !@#$%^&*()-:123",
            "name-with-special-chars !@#$%^&*()-",
            123,
            False,
        ),
        ("version-with-spaces: 123", None, None, True),
        ("version-with-spaces:123 ", None, None, True),
        ("version-with-spaces:1 23", None, None, True),
        ("version-is-not-integer:_123", None, None, True),
        ("version-is-not-integer:v123", None, None, True),
        ("version-is-not-integer:my-version", None, None, True),
        ("more-than-one-colons:1:2", None, None, True),
        ("more-than-one-colons:abc:2", None, None, True),
        ("more-than-one-colons:::2", None, None, True),
        (":1", None, None, True),
        (":", None, None, True),
    ],
)
def test_parse_cluster_compute_name_version(
    cluster_compute_name_version: str,
    expected_name: str,
    expected_version: int,
    error: bool,
):
    if error:
        with pytest.raises(ValueError) as e:
            parse_cluster_compute_name_version(cluster_compute_name_version)
            assert "Invalid compute config name and version" in str(e)
    else:
        assert parse_cluster_compute_name_version(cluster_compute_name_version) == (
            expected_name,
            expected_version,
        )



from unittest.mock import Mock

from anyscale.client.openapi_client.models import (
    AttachMachinePoolToCloudRequest,
    CreateMachinePoolResponse,
    DeleteMachinePoolRequest,
    DetachMachinePoolFromCloudRequest,
    ListMachinePoolsResponse,
    MachinePool,
)
from frontend.cli.anyscale.client.openapi_client.models.cloud import Cloud
from frontend.cli.anyscale.controllers.machine_pool_controller import (
    MachinePoolController,
)


def test_create_machine_pool(mock_auth_api_client) -> None:
    response = CreateMachinePoolResponse(
        machine_pool=MachinePool(
            machine_pool_name="pool1",
            machine_pool_id="mp-123",
            organization_id="org-123",
            cloud_ids=[],
        )
    )
    api_response = Mock()
    api_response.result = response

    machine_pool_controller = MachinePoolController()
    machine_pool_controller.api_client.create_machine_pool_api_v2_machine_pools_create_post = Mock(
        return_value=api_response
    )
    output = machine_pool_controller.create_machine_pool(machine_pool_name="pool1")
    assert output == response


def test_delete_machine_pool(mock_auth_api_client) -> None:
    api_response = Mock()
    api_response.result = ""

    machine_pool_controller = MachinePoolController()
    machine_pool_controller.api_client.delete_machine_pool_api_v2_machine_pools_delete_post = Mock(
        return_value=api_response
    )
    machine_pool_controller.delete_machine_pool(machine_pool_name="pool1")
    machine_pool_controller.api_client.delete_machine_pool_api_v2_machine_pools_delete_post.assert_called_once_with(
        DeleteMachinePoolRequest(machine_pool_name="pool1")
    )


def test_list_machine_pool(mock_auth_api_client) -> None:
    response = ListMachinePoolsResponse(
        machine_pools=[
            MachinePool(
                machine_pool_name="pool1",
                machine_pool_id="mp-123",
                organization_id="org-123",
                cloud_ids=[],
            )
        ]
    )

    api_response = Mock()
    api_response.result = response

    machine_pool_controller = MachinePoolController()
    machine_pool_controller.api_client.list_machine_pools_api_v2_machine_pools_get = Mock(
        return_value=api_response
    )
    output = machine_pool_controller.list_machine_pools()
    assert output == response


def test_attach_machine_pool_to_cloud(mock_auth_api_client) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.api_client.attach_machine_pool_to_cloud_api_v2_machine_pools_attach_post = Mock(
        return_value=""
    )

    find_cloud_by_name_response = Mock()
    find_cloud_by_name_response.result = Cloud(
        id="cld_1",
        name="cloud1",
        provider="aws",
        region="us-east-1",
        credentials={},
        type="",
        creator_id="123",
        created_at="123",
        is_default=True,
    )
    machine_pool_controller.api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post = Mock(
        return_value=find_cloud_by_name_response
    )

    machine_pool_controller.attach_machine_pool_to_cloud(
        machine_pool_name="pool1", cloud="cloud1"
    )
    machine_pool_controller.api_client.attach_machine_pool_to_cloud_api_v2_machine_pools_attach_post.assert_called_once_with(
        AttachMachinePoolToCloudRequest(machine_pool_name="pool1", cloud_id="cld_1")
    )


def test_detach_machine_pool_from_cloud(mock_auth_api_client) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.api_client.detach_machine_pool_from_cloud_api_v2_machine_pools_detach_post = Mock(
        return_value=""
    )

    find_cloud_by_name_response = Mock()
    find_cloud_by_name_response.result = Cloud(
        id="cld_1",
        name="cloud1",
        provider="aws",
        region="us-east-1",
        credentials={},
        type="",
        creator_id="123",
        created_at="123",
        is_default=True,
    )
    machine_pool_controller.api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post = Mock(
        return_value=find_cloud_by_name_response
    )

    machine_pool_controller.detach_machine_pool_from_cloud(
        machine_pool_name="pool1", cloud="cloud1"
    )
    machine_pool_controller.api_client.detach_machine_pool_from_cloud_api_v2_machine_pools_detach_post.assert_called_once_with(
        DetachMachinePoolFromCloudRequest(machine_pool_name="pool1", cloud_id="cld_1")
    )



import click


@click.command(
    name="exec",
    hidden=True,
    help="[DEPRECATED] Execute shell commands in interactive cluster.",
    context_settings={"ignore_unknown_options": True, "allow_extra_args": True,},
)
def anyscale_exec() -> None:
    raise click.ClickException(
        "Warning: `anyscale exec` has been deprecated and no longer works on Anyscale V2. "
        "Please use `anyscale job submit` to run your script as a job in a cluster."
    )



from typing import Optional

import click

from anyscale.cli_logger import BlockLogger
from anyscale.controllers.experimental_integrations_controller import (
    ExperimentalIntegrationsController,
)


log = BlockLogger()  # CLI Logger


@click.group(
    "integration", help="Interact with external integrations on Anyscale.", hidden=True
)
def experimental_integrations_cli() -> None:
    pass


@experimental_integrations_cli.group(
    "wandb", help="Interact with Anyscale W&B integration."
)
def experimental_wandb_cli() -> None:
    pass


@experimental_wandb_cli.command(
    name="enable",
    help="[DEPRECATED] This command is no longer needed to enable the product W&B integration.",
)
@click.option(
    "--cloud-id",
    "-id",
    required=False,
    default=None,
    help="ID of the cloud to enable the W&B integration with.",
)
@click.option(
    "--cloud-name",
    "-n",
    required=False,
    default=None,
    help="Name of the cloud to enable the W&B integration with.",
)
def enable_wandb_integration(
    cloud_id: Optional[str], cloud_name: Optional[str]
) -> None:
    log.warning(
        "`anyscale integration wandb enable` has been deprecated. The product W&B integration "
        "is enabled by default so this command is no longer necessary. We will remove this command "
        "at the end of Q1 2023."
    )

    if not cloud_id and not cloud_name:
        raise click.ClickException(
            "Neither --cloud-id nor --cloud-name were provided. Please specify the "
            "cloud for which to enable the integration through one of these arguments."
        )

    if cloud_id and cloud_name:
        raise click.ClickException(
            "Both --cloud-id and --cloud-name were provided. Please specify the "
            "cloud for which to enable the integration through only one of these arguments."
        )

    integrations_controller = ExperimentalIntegrationsController()
    integrations_controller.enable_wandb_integration(
        cloud_id=cloud_id, cloud_name=cloud_name
    )



# type: ignore

import os
from tempfile import NamedTemporaryFile
from typing import Any, Dict, Optional
from unittest.mock import Mock, mock_open, patch

import boto3
import pytest
import smart_open

from anyscale.client.openapi_client.models.cloud_providers import CloudProviders
from anyscale.client.openapi_client.models.decoratedsession_response import (
    DecoratedsessionResponse,
)
from anyscale.utils.runtime_env import (
    _get_cloud_gs_bucket_from_cloud,
    _get_cloud_s3_bucket_from_cloud,
    autopopulate_runtime_env_for_workspace,
    infer_upload_path_and_rewrite_working_dir,
    override_runtime_env_config,
)
from anyscale.utils.workload_types import Workload


class TestRuntimeEnvOverride:
    """Test 5 cases of overriding runtime env
    1. Runtime env or working_dir is None from non-workspace
        -> If runtime_env is None, autopopulate to empty dict
        -> Else, Do nothing
    2. Working_dir is a remote uri -> Do nothing
    3. Working_dir is local and upload path is defined -> upload and rewrite working_dir
    4. Working_dir is local and upload path is not defined from workspace
        -> Autopopulate working_dir, infer upload path, upload working_dir, and rewrite working_dir
    5. Working_dir is local and upload path is not defined from non-workspace
        -> Infer upload path, upload working_dir, and rewrite working_dir

    Other Cases
    - Working_dir is not specified but upload_path is defined -> Pydantic error
    - Working_dir is a remote uri and upload_path is defined-> Pydantic error
    """

    @pytest.mark.parametrize("runtime_env", [None, {}, {"mock_key": "mock_value"}])
    def test_override_noop(self, runtime_env: Optional[Dict[str, Any]]):
        """ Test 1
        No runtime env or working_dir is defined from non-workspace, then do nothing
        If runtime env is None, then it should be autopopulated to an empty dict
        """
        boto_mock = Mock()
        with patch.multiple(os.path, ismount=Mock(return_value=True)), patch.multiple(
            os, makedirs=Mock()
        ), patch("builtins.open", mock_open()), patch.multiple(
            boto3, client=boto_mock
        ), patch.multiple(
            smart_open, open=Mock()
        ):

            modified_runtime_env = override_runtime_env_config(
                runtime_env=runtime_env,
                anyscale_api_client=Mock(),
                api_client=Mock(),
                workload_type=Mock(),
                compute_config_id=Mock(),
                log=Mock(),
            )

        if runtime_env is None:
            assert modified_runtime_env == {}
        else:
            assert modified_runtime_env == runtime_env

    @pytest.mark.parametrize(
        "is_workspace", [True, False],
    )
    def test_override_with_remote_working_dir(self, is_workspace: bool):
        """ Test 2
        Working_dir is a remote uri -> Return existing runtime env
        """
        runtime_env = {
            "working_dir": "s3://bk-premerge-first-jawfish-artifacts/e2e_tests/job",
        }
        boto_mock = Mock()
        with patch.multiple(os.path, ismount=Mock(return_value=True)), patch.multiple(
            os, makedirs=Mock()
        ), patch("builtins.open", mock_open()), patch.multiple(
            boto3, client=boto_mock
        ), patch.multiple(
            smart_open, open=Mock()
        ):
            if is_workspace:
                mock_os_environ = {
                    "ANYSCALE_SESSION_ID": "fake_session_id",
                    "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id",
                }
                with patch.dict(os.environ, mock_os_environ):
                    modified_runtime_env = override_runtime_env_config(
                        runtime_env=runtime_env,
                        anyscale_api_client=Mock(),
                        api_client=Mock(),
                        workload_type=Mock(),
                        compute_config_id=Mock(),
                        log=Mock(),
                    )
            else:
                modified_runtime_env = override_runtime_env_config(
                    runtime_env=runtime_env,
                    anyscale_api_client=Mock(),
                    api_client=Mock(),
                    workload_type=Mock(),
                    compute_config_id=Mock(),
                    log=Mock(),
                )

        assert modified_runtime_env == runtime_env

    @pytest.mark.parametrize(
        "working_dir", [".", "./subdir/subdir2", "/root_dir/subdir1"],
    )
    @pytest.mark.parametrize(
        "is_workspace", [True, False],
    )
    def test_override_rewrite_working_dir(
        self, working_dir: Optional[str], is_workspace: bool
    ):
        """ Test 3
        Working_dir is a local path and upload path is defined
        1. Upload working_dir
        2. Rewrite working_dir to remote uri
        """
        runtime_env = {
            "working_dir": working_dir,
            "upload_path": "s3://bk-premerge-first-jawfish-artifacts/e2e_tests/job",
        }
        boto_mock = Mock()

        rewritten_runtime_env = {
            "working_dir": "s3://bk-premerge-first-jawfish-artifacts/e2e_tests/job"
        }
        mock_upload_and_rewrite_working_dir = Mock(return_value=rewritten_runtime_env)
        with patch.multiple(
            "anyscale.utils.runtime_env",
            upload_and_rewrite_working_dir=mock_upload_and_rewrite_working_dir,
        ), patch.multiple(os.path, ismount=Mock(return_value=True)), patch.multiple(
            os, makedirs=Mock()
        ), patch(
            "builtins.open", mock_open()
        ), patch.multiple(
            boto3, client=boto_mock
        ), patch.multiple(
            smart_open, open=Mock()
        ):
            if is_workspace:
                mock_os_environ = {
                    "ANYSCALE_SESSION_ID": "fake_session_id",
                    "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id",
                }
                with patch.dict(os.environ, mock_os_environ):
                    modified_runtime_env = override_runtime_env_config(
                        runtime_env=runtime_env,
                        anyscale_api_client=Mock(),
                        api_client=Mock(),
                        workload_type=Mock(),
                        compute_config_id=Mock(),
                        log=Mock(),
                    )
            else:
                modified_runtime_env = override_runtime_env_config(
                    runtime_env=runtime_env,
                    anyscale_api_client=Mock(),
                    api_client=Mock(),
                    workload_type=Mock(),
                    compute_config_id=Mock(),
                    log=Mock(),
                )

            assert modified_runtime_env == rewritten_runtime_env

    @pytest.mark.parametrize(
        "runtime_env", [None, {}, {"working_dir": "job-services-cuj-examples"}]
    )
    def test_runtime_env_override_with_workspace(
        self, runtime_env: Optional[Dict[str, Any]]
    ):
        """ Test 4
        Override runtime env from workspace
        1. Autopopulate the working_dir to local dir if missing
        2. Infer the upload path
        3. Upload working_dir
        4. Rewrite working_dir to remote uri
        """
        workspace_id = "mock_workspace_id"
        cloud_id = "test_cloud_id"
        rewritten_runtime_env = {
            "working_dir": "s3://bucket",
        }
        mock_log = Mock()
        mock_infer_upload_path_and_rewrite_working_dir = Mock(
            return_value=rewritten_runtime_env
        )
        mock_api_client = Mock()

        mock_api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get = Mock(
            return_value=Mock(result=Mock(cloud=Mock(id=cloud_id,)))
        )
        with patch.multiple(os.path, ismount=Mock(return_value=True)), patch.dict(
            os.environ,
            {
                "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": workspace_id,
                "ANYSCALE_SESSION_ID": "mock_session_id",
            },
            clear=True,
        ), patch.multiple(os, makedirs=Mock()), patch(
            "urllib.request", urlretrieve=Mock()
        ), patch(
            "builtins.open", mock_open()
        ), patch.multiple(
            "anyscale.utils.runtime_env",
            infer_upload_path_and_rewrite_working_dir=mock_infer_upload_path_and_rewrite_working_dir,
        ):
            modified_runtime_env = override_runtime_env_config(
                runtime_env=runtime_env,
                anyscale_api_client=Mock(),
                api_client=mock_api_client,
                workload_type=Workload.JOBS,
                compute_config_id=Mock(),
                log=mock_log,
            )

            runtime_env = autopopulate_runtime_env_for_workspace(
                runtime_env=runtime_env, log=Mock()
            )

            mock_infer_upload_path_and_rewrite_working_dir.assert_called_once_with(
                api_client=mock_api_client,
                existing_runtime_env=runtime_env,
                workload_type=Workload.JOBS,
                cloud_id=cloud_id,
                log=mock_log,
                workspace_id=workspace_id,
            )

            assert modified_runtime_env == rewritten_runtime_env

    def test_runtime_env_override_no_workspace(self):
        """ Test 5
        Override runtime env from non-workspace
        1. Infer the upload path
        2. Upload working_dir
        2. Rewrite working_dir to remote uri
        """
        runtime_env = {"working_dir": "job-services-cuj-examples"}
        cloud_id = "test_cloud_id"
        rewritten_runtime_env = {
            "working_dir": "s3://bucket",
        }
        mock_log = Mock()
        mock_infer_upload_path_and_rewrite_working_dir = Mock(
            return_value=rewritten_runtime_env
        )
        mock_anyscale_api_client = Mock()
        mock_api_client = Mock()

        mock_anyscale_api_client.get_compute_template = Mock(
            return_value=Mock(result=Mock(config=Mock(cloud_id=cloud_id,)))
        )
        with patch.multiple(os.path, ismount=Mock(return_value=True)), patch.multiple(
            os, makedirs=Mock()
        ), patch("urllib.request", urlretrieve=Mock()), patch(
            "builtins.open", mock_open()
        ), patch.multiple(
            "anyscale.utils.runtime_env",
            infer_upload_path_and_rewrite_working_dir=mock_infer_upload_path_and_rewrite_working_dir,
        ):
            modified_runtime_env = override_runtime_env_config(
                runtime_env=runtime_env,
                anyscale_api_client=mock_anyscale_api_client,
                api_client=mock_api_client,
                workload_type=Workload.JOBS,
                compute_config_id=Mock(),
                log=mock_log,
            )

            assert modified_runtime_env == rewritten_runtime_env
            mock_infer_upload_path_and_rewrite_working_dir.assert_called_once_with(
                api_client=mock_api_client,
                existing_runtime_env=runtime_env,
                workload_type=Workload.JOBS,
                cloud_id=cloud_id,
                log=mock_log,
            )


@pytest.mark.parametrize(
    (
        "runtime_env",
        "is_workspace",
        "requirements_file_contents",
        "expected_runtime_env",
    ),
    [
        # If not in a workspace, the fields should not be modified.
        [None, False, None, None],
        [{}, False, None, {}],
        [
            {"working_dir": "s3://something.zip", "pip": ["test"]},
            False,
            None,
            {"working_dir": "s3://something.zip", "pip": ["test"]},
        ],
        # If no working_dir is passed, it should default to ".".
        [None, True, None, {"working_dir": "."}],
        [{}, True, None, {"working_dir": "."}],
        # If a working_dir is passed, it should not be altered.
        [
            {"working_dir": "local-path-from-user"},
            True,
            None,
            {"working_dir": "local-path-from-user"},
        ],
        [
            {"working_dir": "s3://remote-path-from-user.zip"},
            True,
            None,
            {"working_dir": "s3://remote-path-from-user.zip"},
        ],
        # If no pip or conda field is specified, the requirements file should be picked up.
        # Also check that the file is read and stripped of comments & whitespace.
        [
            {},
            True,
            "test-pkg1\n test-pkg2==0.1.2\n \n#comment\n ",
            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
        ],
        [
            {},
            True,
            "test-pkg1\n test-pkg2==0.1.2#comment\n ",
            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
        ],
        [
            {},
            True,
            "test-pkg1\n test-pkg2==0.1.2 #comment\n ",
            {"working_dir": ".", "pip": ["test-pkg1", "test-pkg2==0.1.2"]},
        ],
        # If the requirements file is empty (after stripping), the field shouldn't be set.
        [{}, True, "", {"working_dir": "."},],
        [{}, True, "\n#comment\n ", {"working_dir": "."},],
        # If the pip or conda field is specified, it should not be altered.
        [
            {"pip": ["some-other-pkg"]},
            True,
            "test-pkg1\ntest-pkg2\n",
            {"working_dir": ".", "pip": ["some-other-pkg"]},
        ],
        [
            {"conda": ["some-other-pkg"]},
            True,
            "test-pkg1\ntest-pkg2\n",
            {"working_dir": ".", "conda": ["some-other-pkg"]},
        ],
    ],
)
def test_autopopulate_runtime_env_for_workspace(
    runtime_env: Optional[Dict[str, Any]],
    is_workspace: bool,
    requirements_file_contents: Optional[str],
    expected_runtime_env: Optional[Dict[str, Any]],
):
    """Test that working_dir and pip fields are populated properly for workspaces."""
    mock_logger = Mock()
    mock_os_environ = (
        {
            "ANYSCALE_SESSION_ID": "fake_session_id",
            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id",
            "ANYSCALE_WORKSPACE_DYNAMIC_DEPENDENCY_TRACKING": "1",
        }
        if is_workspace
        else {}
    )

    requirements_file_path = None
    if requirements_file_contents is not None:
        f = NamedTemporaryFile("w")
        f.write(requirements_file_contents)
        f.flush()
        requirements_file_path = f.name
    else:
        requirements_file_path = "/some/nonexistent/path"

    with patch.dict(os.environ, mock_os_environ):
        modified_runtime_env = autopopulate_runtime_env_for_workspace(
            runtime_env, mock_logger, requirements_file_path=requirements_file_path,
        )

    assert modified_runtime_env == expected_runtime_env


@pytest.mark.parametrize(
    "workload_type", [Workload.JOBS, Workload.SERVICES, Workload.SCHEDULED_JOBS],
)
@pytest.mark.parametrize(
    "is_workspace", [True, False],
)
@pytest.mark.parametrize(
    "working_dir", [".", "./subdir/subdir2", "/root_dir/subdir1"],
)
@pytest.mark.parametrize(
    "protocol", ["s3", "gs"],
)
def test_infer_upload_path_and_rewrite_working_dir(
    workload_type: Workload, is_workspace: bool, working_dir: str, protocol: str
):
    """This test checks that the upload remote path follows the expected pattern.
    Works for all workloads

    Test every combination
    working_dir [True, False]
    is_workspace [True, False]
    worktype: [jobs, services, scheduled_jobs]
    """
    workspace_id = "test_workspace_id"

    cloud_id = "test_cloud_id"
    org_id = "test_org_id"
    remote_bucket_name = "test_remote_bucket_name"

    # mock api calls utilized for collecting information required to construct the remote path
    mock_api_client = Mock()
    mock_get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get = Mock(
        return_value=DecoratedsessionResponse(result=Mock(cloud=Mock(id=cloud_id)))
    )
    if protocol == "s3":
        mock_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_router_cloud_id_get = Mock(
            return_value=Mock(
                result=Mock(
                    provider=CloudProviders.AWS,
                    cloud_resource=Mock(aws_s3_id=remote_bucket_name),
                )
            )
        )
    elif protocol == "gs":
        mock_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_router_cloud_id_get = Mock(
            return_value=Mock(result=Mock(provider=CloudProviders.GCP, id=cloud_id,))
        )
        mock_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_gcp_router_cloud_id_get = Mock(
            return_value=Mock(
                result=Mock(
                    cloud_resource=Mock(gcp_cloud_storage_bucket_id=remote_bucket_name),
                )
            )
        )
    mock_get_user_info_api_v2_userinfo_get = Mock(
        return_value=Mock(result=Mock(organizations=[Mock(id=org_id)]))
    )
    mock_api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get = (
        mock_get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get
    )
    mock_api_client.get_user_info_api_v2_userinfo_get = (
        mock_get_user_info_api_v2_userinfo_get
    )

    rewritten_runtime_env = {
        "pip": ["requests"],
        "working_dir": f"{protocol}://bucket",
    }
    mock_upload_and_rewrite_working_dir = Mock(return_value=rewritten_runtime_env)
    with patch.multiple(
        "anyscale.utils.runtime_env",
        upload_and_rewrite_working_dir=mock_upload_and_rewrite_working_dir,
    ), patch.multiple(os.path, ismount=Mock(return_value=True)), patch.dict(
        os.environ, {"ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": workspace_id}, clear=True,
    ):

        workspace_id_arg = None
        if is_workspace:
            workspace_id_arg = workspace_id

        infer_upload_path_and_rewrite_working_dir(
            api_client=mock_api_client,
            existing_runtime_env={"working_dir": working_dir},
            cloud_id=cloud_id,
            workspace_id=workspace_id_arg,
            workload_type=workload_type,
            log=Mock(),
        )
    if is_workspace:
        expected_bucket_prefix = f"{protocol}://{remote_bucket_name}/{org_id}/{cloud_id}/workspace_snapshots/{workspace_id}/{workload_type}"
    else:
        expected_bucket_prefix = (
            f"{protocol}://{remote_bucket_name}/{org_id}/{cloud_id}/{workload_type}"
        )
    call_args = mock_upload_and_rewrite_working_dir.call_args
    runtime_env_arg = call_args[0][0]
    assert runtime_env_arg["working_dir"] == working_dir
    assert runtime_env_arg["upload_path"].startswith(expected_bucket_prefix)


@pytest.mark.parametrize(
    ("aws_s3_id", "expected_bucket_name"),
    [
        (None, None),
        (
            "anyscale-production-data-cld-9xr5r1b2g6egh9bwgdi66whsjv",
            "anyscale-production-data-cld-9xr5r1b2g6egh9bwgdi66whsjv",
        ),
        (
            "arn:aws:s3:::anyscale-test-data-cld-n5rtny1bj8pv6gsmb7m5a4l2",
            "anyscale-test-data-cld-n5rtny1bj8pv6gsmb7m5a4l2",
        ),
    ],
)
def test_get_s3_bucket_aws(
    aws_s3_id: Optional[str], expected_bucket_name: Optional[str]
):
    cloud = Mock(provider=CloudProviders.AWS, cloud_resource=Mock(aws_s3_id=aws_s3_id))

    assert _get_cloud_s3_bucket_from_cloud(cloud) == expected_bucket_name


@pytest.mark.parametrize(
    ("gcp_cloud_storage_bucket_id", "expected_bucket_name"),
    [
        (None, None),
        (
            "anyscale-production-data-cld-9xr5r1b2g6egh9bwgdi66whsjv",
            "anyscale-production-data-cld-9xr5r1b2g6egh9bwgdi66whsjv",
        ),
    ],
)
def test_get_gs_bucket_gcp(
    gcp_cloud_storage_bucket_id: Optional[str], expected_bucket_name: Optional[str]
):
    mock_api_client = Mock()
    mock_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_gcp_router_cloud_id_get = Mock(
        return_value=Mock(
            result=Mock(
                cloud_resource=Mock(
                    gcp_cloud_storage_bucket_id=gcp_cloud_storage_bucket_id
                ),
            )
        )
    )

    cloud = Mock(id="test_cloud_id", provider=CloudProviders.GCP,)

    assert (
        _get_cloud_gs_bucket_from_cloud(mock_api_client, cloud) == expected_bucket_name
    )
